diff --git a/src/lib-storage/index/dbox-multi/mdbox-purge.c b/src/lib-storage/index/dbox-multi/mdbox-purge.c
index 00536b5c7b..2cbfc15429 100644
--- a/src/lib-storage/index/dbox-multi/mdbox-purge.c
+++ b/src/lib-storage/index/dbox-multi/mdbox-purge.c
@@ -575,6 +575,10 @@ static int mdbox_altmove_add_files(struct mdbox_purge_context *ctx)
 	uoff_t offset;
 	int ret = 0;
 
+	ARRAY_TYPE(mdbox_map_file_msg) msgs_arr;
+	const struct mdbox_map_file_msg *msgs;
+	unsigned int j, msgs_count = 0;
+
 	/* first add move-to-alt actions */
 	if (array_is_created(&dstorage->move_to_alt_map_uids)) {
 		array_sort(&dstorage->move_to_alt_map_uids, uint32_t_cmp);
@@ -606,6 +610,29 @@ static int mdbox_altmove_add_files(struct mdbox_purge_context *ctx)
 					  POINTER_CAST(action));
 			seq_range_array_add(&ctx->purge_file_ids,
 					    cur_rec.file_id);
+
+			// This code block will move other messages in this file to alternate storage
+			// except the message above. Other word, this way keeps them in the alternate
+			// storage instead of going with the whole file but it will reduce performance.
+			// It's better if we can process separately MOVE_TO_ALT and MOVE_FROM_ALT messages
+			// like what is done with dbox-single.
+			//
+			// Get list of map UIDs that exist in this file
+			i_array_init(&msgs_arr, 128);
+			if (mdbox_map_get_file_msgs(dstorage->map, cur_rec.file_id, &msgs_arr) >= 0) {
+				/* sort messages by their offset */
+				array_sort(&msgs_arr, mdbox_map_file_msg_offset_cmp);
+				msgs = array_get(&msgs_arr, &msgs_count);
+				for (j = 0; j < msgs_count; j++) {
+					// If the message is not affected by any action, move it to the alternate storage
+					if (msgs[j].map_uid != cur_map_uid && hash_table_lookup(ctx->altmoves, POINTER_CAST(msgs[j].map_uid)) == NULL) {
+						action = MDBOX_MSG_ACTION_MOVE_TO_ALT;
+						hash_table_insert(ctx->altmoves, POINTER_CAST(msgs[j].map_uid), POINTER_CAST(action));
+					}
+				}
+			}
+			array_free(&msgs_arr);
+			msgs = NULL;
 		}
 	}
 
